# the args passed to the callback python function is not checked, no error
# is reported if mismatch args are passed, user must make sure by self
import callback, _fmm
fmm_error = 'fmm_error'

def wrap(x):
    xargs = list(extra_args)
    xargs.insert(0, x)
    return apply(funct, tuple(xargs))

def fmin(f, xa, xb, tol=1e-6, *xargs):
    " xmin = fmin(f, xa, xb, tol=1e-6, *xargs) finds the minimum of f(x, xargs) using fmin, a combination of golden section search and successive parabolic interpolation. "
    if len(xargs) == 0:
        cb = callback.callback(f, 'd_dp')
    else:
        global funct, extra_args
    	funct=f; extra_args = xargs
    	cb = callback.callback(wrap, 'd_dp')
    if xa > xb: xa, xb = xb, xa
    return _fmm.fmin_(xa, xb, cb, tol)

def zeroin(f, xa, xb, tol=1e-6, *xargs):
    " xzero = zeroin(f, xa, xb, tol=1e-6, *xargs) finds the zero of f(x, xargs) using zeroin "
    if len(xargs) == 0:
        if f(xa)*f(xb) > 0: 
	     raise fmm_error, 'f(xa), f(xb) must have opposite sign'
        cb = callback.callback(f, 'd_dp')
    else:
        global funct, extra_args
    	funct=f; extra_args = xargs
    	if wrap(xa)*wrap(xb) > 0: 
	    raise fmm_error, 'f(xa, xargs), f(xb, xargs) must have opposite sign'
    	cb = callback.callback(wrap, 'd_dp')
    if xa > xb: xa, xb = xb, xa
    return _fmm.zeroin_(xa, xb, cb, tol)

def quanc8(f, xa, xb, abserr=1e-16, relerr=1e-6, *xargs):
    " [res, errest, nfeval] = quanc8(f, xa, xb, abserr=1e-16, relerr=1e-6, *xargs) integrates f(x, xargs) by quanc8, an automatic adaptive routine based on the 8-panel newton-cotes rule. "
    if len(xargs) == 0:
        cb = callback.callback(f, 'd_dp')
    else:
        global funct, extra_args
    	funct=f; extra_args = xargs
    	cb = callback.callback(wrap, 'd_dp')
    if xa > xb: xa, xb = xb, xa
    [res, errest, nfeval, flag] = _fmm.quanc8_(cb, xa, xb, abserr, relerr)
    if flag != 0:
        raise fmm_error, 'quanc8 fails to converge, flag = ', flag
    else:
        return [res, errest, nfeval]
