      DOUBLE PRECISION FUNCTION CHAOS(INIZ)
C
C  CHAOS  GENERATES A RANDOM SAMPLE FROM ONE OUT OF FOUR POSSIBLE
C PROBABILITY DISTRIBUTIONS USING RANDOM NUMBERS UNIFORMLY
C DISTRIBUTED IN (0,1) GENERATED BY THE FUNCTION  UNIFRD
C
C  INIZ  IS AN INPUT PARAMETER AS FOLLOWS
C
C      INIZ>0          INITIALIZATION WITH SEED  INIZ.
C      INIZ=0          STANDARD GAUSSIAN DISTRIBUTION.
C      INIZ=-1         CAUCHY DISTRIBUTION.
C      INIZ=-2         UNIFORM DISTRIBUTION IN (-1,+1).
C      OTHERWISE       UNIFORM DISTRIBUTION IN ( 0,+1).
C
      DOUBLE PRECISION UNIFRD,PAI,A,B
C
      DATA PAI/3.14159265358979324D0/
C
      IF(INIZ.LE.0) GO TO 10
C
C INITIALIZATION.
C
      CHAOS = UNIFRD(INIZ)
      RETURN
C
  10  CONTINUE
      A = UNIFRD(0)
      IF(INIZ.NE.0) GO TO 20
      B = UNIFRD(0)
C
C GAUSSIAN RANDOM NUMBER BY POLAR METHOD
C
      CHAOS = DSQRT(-2.D0*DLOG(A))*DCOS(PAI*B)
C
      RETURN
  20  CONTINUE
C
C UNIFORM RANDOM NUMBER IN (0,1)
C
      CHAOS = A
C
C CAUCHY RANDOM NUMBER BY INVERSE TRANSFORMATION
C
      IF(INIZ.EQ.(-1)) CHAOS = DSIN(PAI*A)/DCOS(PAI*A)
C
C UNIFORM RANDOM NUMBER IN (-1,+1)
C
      IF(INIZ.EQ.(-2)) CHAOS = 2.D0*A-1.D0
C
      RETURN
      END
