      SUBROUTINE GLOMIP (NPROB, N, X0, XMIN, XMAX)
C
C     THE SUBROUTINE  GLOMIP  PROVIDES THE CODING FOR THE NUMBER
C     OF VARIABLES, THE INITIAL POINT, AND THE OBSERVATION REGION
C     TO BE USED, TOGETHER WITH THE 37 TEST FUNCTIONS GIVEN BY
C     SUBROUTINE  GLOMTF , TO DEFINE 37 TEST PROBLEMS FOR GLOBAL
C     MINIMIZATION SOFTWARE.
C
C     THE SUBROUTINE  GLOMIP  RETURNS IN  N , X0 , AND  XMIN ,
C     XMAX  THE NUMBER OF VARIABLES, THE INITIAL POINT, AND THE
C     BOUNDARIES OF THE OBSERVATION REGION.
C
C     CALLING STATMENT
C        CALL GLOMIP (NPROB, N, X0, XMIN, XMAX)
C
C     DESCRIPTION OF THE CALL PARAMETERS
C        (THE FORTRAN IMPLICIT TYPE DEFINITION FOR INTEGERS IS USED.
C        ALL NON-INTEGER PARAMETERS ARE DOUBLE-PRECISION).
C
C     NPROB   IS THE (INPUT) NUMBER OF THE TEST PROBLEM TO BE
C             CONSIDERED
C     N       IS THE (OUTPUT) NUMBER OF VARIABLES (DIMENSION) OF
C             THE PROBLEM
C     XMIN , XMAX   ARE THE (OUTPUT) N-VECTORS CONTAINING THE LEFT
C             AND RIGHT BOUNDARIES OF THE OBSERVATION REGION
C             DEFINED BY THE POINTS  X = (X1,...,XN)  SUCH THAT
C                XMIN(I) .LE. X(I) .LE. XMAX(I),  I = 1,...,N
C
      DOUBLE PRECISION X0,XMIN,XMAX
      DOUBLE PRECISION V0,VMIN,VMAX
C
      DIMENSION X0(1),XMIN(1),XMAX(1)
C
      GO TO (10,20,30,40,50,60,70,80,90,100,110,120,130,140,150,
     1       160,170,180,190,200,210,220,230,240,250,260,270,280,
     2       290,300,310,320,330,340,350,360,370),NPROB
C
C    1  A FOURTH-ORDER POLYNOMIAL  (N = 1)
C
   10 CONTINUE
      N = 1
      V0 = 1.0D0
      VMIN = -10.D0
      VMAX = 10.D0
      GO TO 800
C
C    2  GOLDSTEIN SIXTH ORDER POLYNOMIAL  (N = 1)
C
   20 CONTINUE
      N = 1
      V0 = 0.D0
      VMIN = -4.D0
      VMAX = 4.D0
      GOTO 800
C
C    3  ONE-DIMENSIONAL PENALIZED SHUBERT FUNCTION  (N = 1)
C
   30 CONTINUE
      N = 1
      V0 = 0.D0
      VMIN = -10.D0
      VMAX = 10.D0
      GOTO 800
C
C    4  A FOURTH ORDER POLYNOMIAL IN THO VARIABLES  (N = 2)
C
   40 CONTINUE
      N = 2
      X0(1) = 1.D0
      X0(2) = 0.D0
      VMIN = -10.D0
      VMAX = 10.D0
      GO TO 700
C
C    5  A FUNCTION WITH A SINGLE ROW OF LOCAL MINIMA  (N = 2)
C
   50 CONTINUE
      N = 2
      X0(1) = -3.D0
      X0(2) = 0.D0
      XMIN(1) = -15.D0
      XMAX(1) = 25.D0
      XMIN(2) = -5.D0
      XMAX(2) = 15.D0
      RETURN
C
C    6  SIX-HUMP CAMEL FUNCTION  (N = 2)
C
   60 CONTINUE
      N = 2
      V0 = 0.D0
      XMIN(1) = -3.D0
      XMAX(1) = 3.D0
      XMIN(2) = -2.D0
      XMAX(2) = 2.D0
      GO TO 900
C
C    7  TWO-DIMENSIONAL PENALIZED SHUBERT FUNCTION, BETA = 0  (N = 2)
C
   70 CONTINUE
C
C    8  TWO-DIMENSIONAL PENALIZED SHUBERT FUNCTION, BETA = 1/2  (N = 2)
C
   80 CONTINUE
C
C    9  TWO-DIMENSIONAL PENALIZED SHUBERT FUNCTION, BETA = 1  (N = 2)
C
   90 CONTINUE
      N = 2
      V0 = 0.D0
      VMIN = -10.D0
      VMAX = 10.D0
      GO TO 800
C
C   10  A FUNCTION WITH THREE ILL-CONDITIONED MINIMA, A=10  (N = 2)
C
  100 CONTINUE
C
C   11  A FUNCTION WITH THREE ILL-CONDITIONED MINIMA, A=10**2  (N = 2)
C
  110 CONTINUE
C
C   12  A FUNCTION WITH THREE ILL-CONDITIONED MINIMA, A=10**3  (N = 2)
C
  120 CONTINUE
C
C   13  A FUNCTION WITH THREE ILL-CONDITIONED MINIMA, A=10**4  (N = 2)
C
  130 CONTINUE
C
C   14  A FUNCTION WITH THREE ILL-CONDITIONED MINIMA, A=10**5  (N = 2)
C
  140 CONTINUE
C
C   15  A FUNCTION WITH THREE ILL-CONDITIONED MINIMA, A=10**6  (N = 2)
C
  150 CONTINUE
      N = 2
      X0(1) = 0.D0
      X0(2) = 0.D0
      XMIN(1) = -10.D0
      XMAX(1) = 10.D0
      XMIN(2) = -100.D0
      XMAX(2) = 100.D0
      RETURN
C
C   16  GOLDSTEIN-PRICE FUNCTION  (N = 2)
C
  160 CONTINUE
      N = 2
      V0 = 1.D0
      VMIN = -2.D0
      VMAX = 2.D0
      GO TO 800
C
C   17  PENALIZED BRANIN FUNCTION  (N = 2)
C
  170 CONTINUE
      N = 2
      X0(1) = 2.5D0
      X0(2) = 7.5D0
      XMIN(1) = -5.D0
      XMAX(1) = 10.D0
      XMIN(2) = 0.D0
      XMAX(2) = 15.D0
      RETURN
C
C   18  PENALIZED SHEKEL FUNCTION, M = 5  (N = 4)
C
  180 CONTINUE
C
C   19  PENALIZED SHEKEL FUNCTION, M = 7  (N = 4)
C
  190 CONTINUE
C
C   20  PENALIZED SHEKEL FUNCTION, M = 10  (N = 4)
C
  200 CONTINUE
      N = 4
      V0 = 9.D0
      VMIN = 0.D0
      VMAX = 10.D0
      GO TO 800
C
C   21  PENALIZED THREE-DIMENSIONAL HARTMAN FUNCTION  (N = 3)
C
  210 CONTINUE
      N = 3
      GO TO 225
C
C   22  PENALIZED SIX-DIMENSIONAL HARTMAN FUNCTION  (N = 6)
C
  220 CONTINUE
      N = 6
  225 CONTINUE
      V0 = 0.5D0
      VMIN = 0.D0
      VMAX = 1.D0
      GO TO 800
C
C   23  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 1  (N = 2)
C
  230 CONTINUE
      N = 2
      GO TO 315
C
C   24  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 1  (N = 3)
C
  240 CONTINUE
      N = 3
      GO TO 315
C
C   25  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 1  (N = 4)
C
  250 CONTINUE
      N = 4
      GO TO 315
C
C   26  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 2  (N = 5)
C
  260 CONTINUE
      N = 5
      GO TO 315
C
C   27  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 2  (N = 8)
C
  270 CONTINUE
      N = 8
      GO TO 315
C
C   28  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 2  (N = 10)
C
  280 CONTINUE
      N = 10
      GO TO 315
C
C   29  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 3, RANGE = 10   (N = 2)
C
  290 CONTINUE
      N = 2
      GO TO 315
C
C   30  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 3, RANGE = 10   (N = 3)
C
  300 CONTINUE
      N = 3
      GO TO 315
C
C   31  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 3, RANGE = 10   (N = 4)
C
  310 CONTINUE
      N = 4
C
  315 CONTINUE
      V0 = 0.D0
      VMIN = -10.D0
      VMAX = 10.D0
      GO TO 800
C
C   32  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 3, RANGE = 5   (N = 5)
C
  320 CONTINUE
      N = 5
      GO TO 345
C
C   33  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 3, RANGE = 5   (N = 6)
C
  330 CONTINUE
      N = 6
      GO TO 345
C
C   34  PENALIZED LEVY-MONTALVO FUNCTION, TYPE 3, RANGE = 5  (N = 7)
C
  340 CONTINUE
      N = 7
C
  345 CONTINUE
      V0 = 0.D0
      VMIN = -5.D0
      VMAX = 5.D0
      GO TO 800
C
C   35  A FUNCTION WITH A SINGLE CUSP-SHAPED MINIMUM  (N = 5)
C
  350 CONTINUE
      N = 5
      V0 = 1000.D0
      VMIN = -20000.D0
      VMAX = 10000.D0
      GO TO 800
C
C   36  A FUNCTION WITH A SMALL-ATTRACTION-REGION GLOBAL MINIMUM (N = 2)
C
  360 CONTINUE
      N = 2
      X0(1) = 0.D0
      X0(2) = 100.D0
      VMIN = -1000.D0
      VMAX = 1000.D0
      GO TO 700
C
C   37  A FUNCTION WITH A SMALL-ATTRACTION-REGION GLOBAL MINIMUM (N = 5)
C
  370 CONTINUE
      N = 5
      X0(1) = 0.D0
      X0(2) = 0.D0
      X0(3) = 0.D0
      X0(4) = 0.D0
      X0(5) = 10.D0
      VMIN = -100.D0
      VMAX = 100.D0
      GO TO 700
C
  700 CONTINUE
      DO 710 I = 1,N
         XMIN(I) = VMIN
         XMAX(I) = VMAX
  710 CONTINUE
      RETURN
C
  800 CONTINUE
      DO 810 I = 1,N
         XMIN(I) = VMIN
         XMAX(I) = VMAX
  810 CONTINUE
C
  900 CONTINUE
      DO 910 I = 1,N
         X0(I) = V0
  910 CONTINUE
      RETURN
      END
