      SUBROUTINE SSTEP(KTIM,NDIM,X,H,EPS,DX,IE,F)
C
C  THE BASIC TIME-INTEGRATION STEP FOR A GIVEN TRAJECTORY IS PERFORMED
C  BY THE SUBROUTINE  STEP  WHICH
C   - CALLS THE FUNCTION  FUNCT0  TO COMPUTE THE VALUE OF  F
C   - CALLS THE SUBROUTINE  UNITRV  TO COMPUTE THE RANDOM DIRECTION
C        ALONG WHICH THE DIRECTIONAL DERIVATIVE  GAM/N  IS TO BE
C        COMPUTED
C   - CALLS THE SUBROUTINE  DERFOR (OR DERCEN ) TO COMPUTE THE FORWARD-
C        (OR CENTRAL-) DIFFERENCES DIRECTIONAL DERIVATIVE  GAM/N
C   - CALLS THE SUBROUTINE  NEWH  TO ACCEPT OR REJECT THE FIRST HALF-
C        STEP AND OBTAIN AN UPDATED VALUE FOR  H
C   - CALLS THE SUBROUTINE  CUMSCA  TO UPDATE THE CUMULATED SCALING DATA
C   - UPDATES THE SPATIAL DISCRETIZATION INCREMENT  DX  BASED ON THE
C        RESULTS OF CALLING THE FUNCTION  ITOLCH
C   - CALLS THE SUBROUTINE  GAUSRV  TO PERFORM THE SECOND HALF-STEP.
C
      DOUBLE PRECISION X,H,EPS,DX,F
      DOUBLE PRECISION DFDX,DFDXV,DXMAX,DXMIN
      DOUBLE PRECISION EPSR,FS,FV,FVS,HMINS,HR,HS
      DOUBLE PRECISION RCD,RDX,STF,TOLABS,TOLRA,TOLRI
      DOUBLE PRECISION W,XP
      DOUBLE PRECISION FUNCT0
      DIMENSION X(NDIM)
      DIMENSION W(100),XP(100)
      DATA RDX/1.D-4/
      DATA DXMIN/1.D-35/
      DATA DXMAX/1.D3/
      DATA HR/1.D-1/
      DATA HMINS/1.D-30/
      DATA STF /100.D0/
      DATA RCD/2.D0/
      DATA TOLRI/1.D-5/
      DATA TOLRA/1.D-11/
      DATA TOLABS/0.D0/
C
      IEC = 0
      FV = F
  10  CONTINUE
  20  CONTINUE
C
C TAKE A RANDOM DIRECTION FOR THE DIRECTIONAL DERIVATIVE
      CALL UNITRV(NDIM,W)
C
C COMPUTE FORWARD-DIFFERENCE DERIVATIVE
      CALL DERFOR(NDIM,X,FV,DX,W,DFDX)
C
C TRY THE FIRST HALF-STEP
      DO 30 IC = 1,NDIM
         XP(IC) = X(IC)-H*W(IC)*DFDX*DBLE(FLOAT(NDIM))
  30     CONTINUE
      HS = H
      F = FUNCT0(NDIM,XP)
      FVS = FV+DX*DABS(DFDX)
C
C UPDATE STEPLENGTH  H  AND ACCEPT OR REJECT THE HALF-STEP
      CALL NEWH(KTIM,FVS,F,H,IE,IEC)
      IF(IEC.LE.0)GO TO 50
      IE = IE-1
      IEC = IEC-1
      H = HS
      DFDXV = DFDX
C
C COMPUTE CENTRAL-DIFFERENCES DERIVATIVE
      CALL DERCEN(NDIM,X,FV,DX,W,DFDX)
C
C TRY AGAIN THE FIRST HALF-STEP
      DO 40 IC = 1,NDIM
         XP(IC) = X(IC)-H*W(IC)*DFDX*DBLE(FLOAT(NDIM))
  40     CONTINUE
      F = FUNCT0(NDIM,XP)
      FVS = FV+DX*DABS(DFDXV-DFDX)
C
C
C UPDATE STEPLENGTH  H  AND ACCEPT OR REJECT THE HALF-STEP
      CALL NEWH(KTIM,FVS,F,H,IE,IEC)
C
C UPDATE CUMULATED PAST SCALING DATA
      IF(IEC.GE.1)CALL CUMSCA(NDIM,W,DFDX)
      IF(IEC.GE.1)GO TO 10
      DX = DX*RDX
C
C FIRST HALF-STEP ACCEPTED
  50  CONTINUE
C
C UPDATE CUMULATED PAST SCALING DATA
      CALL CUMSCA(NDIM,W,DFDX)
      FS = FV+DX*DABS(DFDX)
      IF(ITOLCH(FS,FV,TOLRI,TOLABS).EQ.0)DX = DX/RCD
      IF(ITOLCH(FS,FV,TOLRA,TOLABS).GT.0)DX = DX*RCD
      EPSR = DSQRT(H)*EPS
C
C TAKE A SAMPLE INCREMENT OF THE WIENER PROCESS
      CALL GAUSRV(NDIM,W)
C
C TRY THE SECOND HALF-STEP
      DO 60 IC = 1,NDIM
         XP(IC) = XP(IC)+EPSR*W(IC)
  60     CONTINUE
      F = FUNCT0(NDIM,XP)
C
C ACCEPT OR REJECT THE COMPLETE STEP
      IF (F-FV.LE.EPS*EPS*STF) GO TO 70
      H = H*HR
      IE = IE+1
      IF (H.GT.HMINS) GO TO 20
C
C STEP ACCEPTED
  70  CONTINUE
      DO 80 IC = 1,NDIM
         X(IC) = XP(IC)
  80     CONTINUE
      DX = DMIN1(DX,DXMAX)
      DX = DMAX1(DX,DXMIN)
C
      RETURN
      END
