/* Modified by M Xu 3/20/99 */
#include "Python.h"
#include "callback.h"
#include "arrayobject.h"
#include <varargs.h>
#include <stdio.h>

/* stores the Python function which is wrapped */
static PyObject *my_callback = NULL;
/* stores the C/Fortran function or wrapper for a Python function, external */
userfunc *userfn[BUF_SIZE];

/****************************************/
/**** User Defined Functions ************/
/****************************************/
/* a sample wrapper for d_dp */
static double d_dp(va_alist) va_dcl
{
    va_list ap;
    PyObject *args, *res; 
    double *x, r;

    /* fetch the arguments from the va_alist */
    va_start(ap);
    x = va_arg(ap, double*);
    va_end(ap);

    /* build the arguments from C to Python */
    args = Py_BuildValue("(d)", *x);

    /* Call the Python Function */
    res = PyEval_CallObject(my_callback, args);
    Py_DECREF(args); 
    if ( res == NULL ) {
        perror("Callback error: expect a numeric value");
    	return 0;
    }
    if ( !PyFloat_Check(res) ) {
        Py_XDECREF(res);
        perror("Callback error: expect a numeric value");
        return 0;
    }

/* get the C result from Python res */ 
    r = PyFloat_AsDouble(res);

    Py_XDECREF(res);
    return r;
}

/* a sample wrapper for d_d */
static double d_d(double x) 
{
    double r;
    PyObject *args, *res;

    /* build the arguments from C to Python */
    args = Py_BuildValue("(d)", x);

    /* Call the Python Function */
    res = PyEval_CallObject(my_callback, args);
    Py_DECREF(args); 
    if ( res == NULL ) {
        perror("Callback error: expect a numeric value");
    	return 0;
    }
    if ( !PyFloat_Check(res) ) {
        Py_XDECREF(res);
        perror("Callback error: expect a numeric value");
        return 0;
    }

    /* get the C result from Python res */ 
    r = PyFloat_AsDouble(res);

    Py_XDECREF(res);
    return r;
}

/* the callback wrapper for BTRK, the Boender-Timmer-Rinnoy Kan code */
static void btrk(double *x, double *f, int *nparam, int *i)
{
    PyObject *ap, *arg, *res;
    /* the convention for btrk is 1-offset, cf opt.c */
    ap =  PyArray_FromDimsAndData(1, nparam, PyArray_DOUBLE, (char *)(x+1));
    arg = Py_BuildValue("(O)", ap);
    res = PyEval_CallObject(my_callback, arg); 
    Py_DECREF(ap);
    Py_DECREF(arg);
    if ( res == NULL ) {
    	return NULL;
    }
    if ( !PyFloat_Check(res) ) {
        Py_XDECREF(res);
        perror("Callback error: expect a numeric value");
        return NULL;
    }
   
    *f = PyFloat_AsDouble(res);
    Py_XDECREF(res);
}

/* the callback wrapper for SIGMA */
static double sigma(int *n, double *x)
{
    PyObject *ap, *arg, *res;
    double r;
    
    /* the convention for btrk is 1-offset, cf opt.c */
    ap =  PyArray_FromDimsAndData(1, n, PyArray_DOUBLE, (char *)x);
    arg = Py_BuildValue("(O)", ap);
    res = PyEval_CallObject(my_callback, arg); 
    Py_DECREF(ap);
    Py_DECREF(arg);
    if ( res == NULL ) {
        perror("Callback error: got None from Python callback");
    	return 0;
    }
    if ( !PyFloat_Check(res) ) {
        Py_XDECREF(res);
        perror("Callback error: got not a float res");
        return 0;
    }
   
    r = PyFloat_AsDouble(res);
    Py_XDECREF(res);
    return r;
}

/* external function in some library */
extern void fcn2_();

/* a dummy MAIN__ function needed when linking fortran functions */
int MAIN__(void)
{ 
    perror("Should never be called.");
    return 0;
}

/* a sample subroutine written for dodr */
static void dodr(va_alist) va_dcl
{
    PyObject *bartn, *res;
    int i; 
    double *dp;
    int *n, *m, *np, *nq;
    int *ldn, *ldm,*ldnp;
    int *ifixb, *ifixx, *ldifx, *ideval, *istop;
    int ndims[3];
    double *beta, *xplusd, *f, *fjacb, *fjacd;
    PyObject *Pbeta, *Pxplusd, *Pf, *Pfjacb, *Pfjacd,*Pistop,*Pifixb,*Pifixx;

    /* read in the arguments */
    va_list AP; va_start(AP);  
    n = va_arg(AP, int*);
    m = va_arg(AP, int*);
    np = va_arg(AP, int*);
    nq = va_arg(AP, int*);
    ldn = va_arg(AP, int*);
    ldm = va_arg(AP, int*);
    ldnp = va_arg(AP, int*);
    beta = va_arg(AP, double*);
    xplusd = va_arg(AP, double*);
    ifixb = va_arg(AP, int*);
    ifixx = va_arg(AP, int*);
    ldifx = va_arg(AP, int*);
    ideval = va_arg(AP, int*);
    f = va_arg(AP, double*);
    fjacb = va_arg(AP, double*);
    fjacd = va_arg(AP, double*);
    istop = va_arg(AP, int*);
    va_end(AP);
    
    Pbeta = PyArray_FromDimsAndData(1, np, PyArray_DOUBLE, (char *)beta);
    ndims[0] = *m; ndims[1] = *ldn;
    Pxplusd = PyArray_FromDimsAndData(2, ndims, PyArray_DOUBLE, (char *)xplusd);
    ndims[0] = *nq; ndims[1] = *ldn;
    Pf = PyArray_FromDimsAndData(2, ndims, PyArray_DOUBLE, (char *)f);
    ndims[0] = 1;
    Pistop = PyArray_FromDimsAndData(1, ndims, PyArray_LONG, (char *)istop);
    Pifixb = PyArray_FromDimsAndData(1, np, PyArray_LONG, (char *)ifixb);
    ndims[1] = *ldifx; ndims[0] = *m;
    Pifixx = PyArray_FromDimsAndData(2, ndims, PyArray_LONG, (char *)ifixx);
    /* Note: fjacb and fjacd are not used in the implicit model.  So they are
     * note passed to the python function.  The integer zero is passed instead 
     * to act as a placeholder in the argument list
     */
    /*
     ndims[0] = *nq; ndims[1] = *ldnp; ndims[2] = *ldn;
     Pfjacb = PyArray_FromDimsAndData(3,ndims,PyArray_DOUBLE,(char *)fjacb);
     ndims[0] = *nq; ndims[1] = *ldm; ndims[3] = *ldn;
     Pfjacd = PyArray_FromDimsAndData(3,ndims,PyArray_DOUBLE,(char *)fjacd);
    */
    bartn =  Py_BuildValue("(iiiiiiiOOOOiiOiiO)",*n,*m,*np,*nq,*ldn,*ldm,*ldnp,Pbeta,Pxplusd,Pifixb,Pifixx,*ldifx,*ideval,Pf,0,0,Pistop);

    res = PyEval_CallObject(my_callback, bartn); 

    Py_DECREF(Pbeta); Py_DECREF(Pxplusd); Py_DECREF(Pf);
    Py_DECREF(Pistop); Py_DECREF(Pifixb); Py_DECREF(Pifixx);
    Py_XDECREF(bartn); Py_XDECREF(res); 
}    


/****************************************/
/**** Utility Functions      ************/
/****************************************/

/* add a function to a dictionary */
static int add_function(void *fp, char *nm, char *ds, PyObject *d)
{
    static int ind=-1;
    PyObject *name, *tup;

    ind++;
    if (ind > BUF_SIZE) { 
        perror("The userfunction buffer is full.\n"); 
        return 1;
    }
    /* setup the userfn */
    userfn[ind] = (userfunc *) malloc(sizeof(userfunc));
    userfn[ind]->fp = fp;
    strcpy(userfn[ind]->name, nm);
    strcpy(userfn[ind]->des, ds);
    userfn[ind]->pyfn = NULL;

    /* add into the dictionary */
    name = PyString_FromString(nm);
    tup = Py_BuildValue("(is)", ind, ds);
    PyObject_SetItem(d, name, tup);
    /* Releasing reference counts */
    Py_DECREF(name);
    Py_DECREF(tup);
    return 0;
}

/* userfn_NameList() returns a python dictionary.  The dictionary should
 * help explain the function to index mapping and give a brief
 * description of the functions
 *
 * userfn_NameList:  name --> (index, des)
 *
 */
extern PyObject *userfn_NameList()
{   
    PyObject *d;
    d = PyDict_New();
    if (d == NULL) {
        perror("Can not initialize a Python dictionary.\n");
        return NULL;
    }
    /* add fortran function fcn2 */
    if ( add_function(fcn2_, "fcn2_", "A Fortran callback for Example 2 from ODRPACK -- implicit model.  Use to fit ellipses to x-y data", d) ) return NULL;

    /* add a wrapper for python callback equvalent to fcn2 */
    if ( add_function(dodr, "dodr", "A wrapper of Python callback for Example 2 from ODRPACK -- implicit model.  Use to fit ellipses to x-y data", d) ) return NULL;

    /* add a wrapper of type double (double *) for Python callback d=PyFun(d) */
    if ( add_function(d_dp, "d_dp", "A wrapper of form double(double *) for Python callback: double = f(double)", d) ) return NULL;

    /* add a Python function which receives a double, and returns a double */
    if ( add_function(d_d, "d_d", "A wrapper of form double(double) for Python callback: double = f(double)", d) ) return NULL;

    /* add a Python function wrapper for btrk */
    if ( add_function(btrk, "GLOBAL", "A wrapper of Python callback: double = f(NumPy_Array) for btrk", d) ) return NULL;

    /* add a Python function wrapper for SIGMA */
    if ( add_function(sigma, "SIGMA", "A wrapper of Python callback: double = f(NumPy_Array) for SIGMA", d) ) return NULL;

    return d;
}

/* client calls this to get a suitable pointer to 
 * a C/Fortran callback or a wrapper for Python callback 
 */  
extern void *getCallbackFn(PyObject *cb)
{       
    PyObject *index;
    int i;
    if ((index = PyObject_GetAttrString(cb,"index"))==NULL) return NULL;
    i = PyInt_AsLong(index);
    Py_DECREF(index);
    if ((i < 0) || (i >= BUF_SIZE)) return NULL;
    my_callback = userfn[i]->pyfn;
    return userfn[i]->fp;
}
